﻿using AdventOfCode.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AdventOfCode._2022.Day10
{
    public class CommunicationCPU
    {
        private static readonly int[] CHECK_CYCLES = new int[]{20, 60, 100, 140, 180, 220};
        private int _registerX;
        private int _cycle;
        private int _sumOfStregnth;
        private char[] _crtScreen;
        private AOCVersion _version;
        public CommunicationCPU(AOCVersion version)
        {
            _version = version;
            _registerX = 1;
            _cycle = 0;
            _sumOfStregnth = 0;
            _crtScreen = Enumerable.Repeat(' ', 240).ToArray();
        }
        public string PrintCRTScreen()
        {
            var sb = new StringBuilder();
            for (int i = 1; i <= _crtScreen.Length; i++)
            {
                sb.Append(_crtScreen[i - 1] + " ");
                if (i % 40 == 0)
                    sb.AppendLine();
            }
            sb.AppendLine();
            return sb.ToString();
        }
        public int GetCycle() { return _cycle; }
        public int GetRegisterValue() { return _registerX; }
        public int GetSignalStrength()
        {
            return _sumOfStregnth;
        }

        public void ExecuteInstruction(string instruction)
        {
            var command = Enum.Parse<CommandType>(instruction.Split(" ")[0].ToUpper());
            if (command == CommandType.NOOP)
            {
                CheckCycle();
            }else if (command == CommandType.ADDX)
            {
                var amount = int.Parse(instruction.Split(" ")[1]);
                CheckCycle();
                CheckCycle();
                _registerX += amount;
            }
        }

        private void CheckCycle()
        {
            if (_version == AOCVersion.B)
            {
                _crtScreen[_cycle] = Math.Abs((_cycle%40) - _registerX) <= 1 ? '#' : '.';
            }
            _cycle++; //Increase it after we draw CRT screen but before we do the cycle check for part A
            if (_version == AOCVersion.A && CHECK_CYCLES.Any(x => x == _cycle))
            {
                _sumOfStregnth += _cycle * _registerX;
            }
        }

    }

    enum CommandType
    {
        NOOP,
        ADDX
    }
}
